#!/usr/local/bin/perl

############################################
##                                        ##
##               WebSearch                ##
##           by Darryl Burgdorf           ##
##       (e-mail burgdorf@awsd.com)       ##
##                                        ##
##             version:  2.01             ##
##        last modified:  02/21/01        ##
##           copyright (c) 2001           ##
##                                        ##
##    latest version is available from    ##
##        http://awsd.com/scripts/        ##
##                                        ##
############################################

# COPYRIGHT NOTICE:
#
# Copyright 2001 Darryl C. Burgdorf.  All Rights Reserved.
#
# This program is being distributed as shareware.  It may be used and
# modified by anyone, so long as this copyright notice and the header
# above remain intact, but any usage should be registered.  (See the
# documentation file for registration information.)  By using this
# program you agree to indemnify Darryl C. Burgdorf from any liability.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  Obtain permission before redistributing this
# program over the Internet or in any other medium.  In all cases
# copyright and header must remain intact.

# VERSION HISTORY:
#
# 2.01  02/21/01  Added $HourOffset configuration variable
#                 Moved SSI parsing from "meta" to "header" file
# 2.00  02/07/01  SCRIPT NOW REQUIRES PERL 5!
#                 Files list only if *no* form, not just if empty form
#                 Allowed designation of keywords in QUERY_STRING
#                 Allowed for optional use of search index file
#                 Allowed search of "extra" (foreign) characters
#                 Made searches more compatible with WebBBS 4.XX
#                 Added "search by date" capability
#                 Improved stripping of <SCRIPT> contents
#                 Made description length configurable
#                 Made search form somewhat more configurable
#                 Improved "list navigation" buttons
#                 Added $meta_file configuration variable
#                 Allowed for SSI includes in header & footer
#                 Made many other minor corrections and updates
# 1.11  02/19/98  Fixed bug in "hits per page" maintenance
#                 Made use of descriptions optional
#                 Eliminated <SCRIPT> contents from searches
# 1.10  02/12/98  FIRST SHAREWARE RELEASE
#                 Allowed for searching of non-alphanumeric characters
#                 Descriptions are now drawn from page text w/o METAs
#                 Dumped "relevance" for keyword matches & filesize
#                 Added optional "A NAME" tag splits to results listing
#                 Added total kilobytes to main file list display
#                 Made "hits per page" user (rather than admin) option
#                 Added $bodyspec, $header_file and $footer_file
#                 Improved appearance of "search results" page
#                 Corrected year refs from 19XX to 1900+XX
#                 Added comma delimiting to large numbers
#                 Numerous other subtle but worthwhile changes!
# 1.08  12/12/97  Made search/display of METAs optional
#                 Eliminated "minimum length" for search terms
#                 Added path/URL replace to title as well as address
#                 Restored stripping of quote marks from search terms
# 1.07  08/06/97  Squashed "display by date" bug
#                 Fixed "Next X" if last set less than X
# 1.06  08/02/97  Added use of META description tags in search output
#                 Replaced UNIX-specific "ls" with calls to "find.pl"
#                 Allowed distinction of "only HTML" and "all text" dirs
#                 Allowed full recursion in directory searches
#                 Added ability to simply list files in search
#                 Added optional display by date rather than relevance
#                 Corrected $avoid format to allow regexs
# 1.05  05/25/97  Fixed bug in "Next/Previous Set" forms
# 1.04  05/21/97  Added META tags to searched material
#                 Revised search directory definition methodology
#                 Added ability to search within multiple URLs
#                 Set script to display only X matches per page
#                 Added "as a phrase" option to boolean choices
#                 Changed "value" count to "relevance" computation
#                 Eliminated incomplete "choose directory" option
# 1.03  04/03/97  Added ALT text to searched material
#                 Added display of total number of files searched
#                 Added $avoid to designate files not to be searched
#                 Fixed bug in the way titles are obtained
#                 Fixed bug introduced by "minor code shuffling"
# 1.02  02/17/97  Fixed bug in Get_Date subroutine
# 1.01  02/07/97  Minor code shuffling
# 1.00  02/03/97  Initial "public" release

##################################
# DEFINE THE FOLLOWING VARIABLES #
##################################

@dirs = ('/usr/www/foo/scripts/dir1/','/usr/www/foo/scripts/dir2/*');

%webbbs4_dirs = ();

$DBMType = 0;

$searchindex = "";

$avoid = '(\.backup|\.cgi|\.pl|\.txt)';

$cgiurl = 'http://www.foo.com/cgi-bin/websearch.pl';

$basepath = '/usr/www/foo/scripts/';
$baseurl = 'http://www.foo.com/scripts/';

# %otherurls = (
#   '/usr/www/foo/scripts/dir2/sub1/',
#   'http://www.foo.com/cgi-bin/some.cgi?access=',
#   '/usr/www/foo/scripts/dir2/sub2/',
#   'http://www.foo.com/scripts/dir2/sub2/another.cgi?read='
#   );

$AllowDateSearch = 1;
$DisplayByDate = 0;

%extrachars = ();

$NoMETAs = 0;

$UseDescs = 1;
$DescLength = 500;

$SplitNames = 0;

$bodyspec = "BGCOLOR=\"#ffffff\" TEXT=\"#000000\"";
$meta_file = "";
$header_file = "/usr/www/foo/header.txt";
$footer_file = "/usr/www/foo/footer.txt";
$SSIRootDir = "";

$PrintNewForm = 1;
$FormExplanation = "The search terms you input do not have to be complete words. <BR>&quot;Wash,&quot; for example, will match occurrences of wash, washer, Washington, etc.<BR>Do not include asterisks or other non-alphanumeric characters in your search terms<BR>unless you actually want them included (as with &quot;C++&quot;) as part of your search.";

$HourOffset = 0;

###################################
# CHANGE NOTHING BELOW THIS LINE! #
###################################

$version = "2.01";

use Fcntl;
BEGIN { @AnyDBM_File::ISA = qw (DB_File GDBM_File SDBM_File ODBM_File NDBM_File) }
use AnyDBM_File;
umask (0111);
require "find.pl";

@day = (Sun,Mon,Tue,Wed,Thu,Fri,Sat);
@month = (Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec);

if ($ENV{'CONTENT_LENGTH'}) {
	read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
	@pairs = split(/&/, $buffer);
	foreach $pair (@pairs){
		($name, $value) = split(/=/, $pair);
		$name =~ tr/+/ /;
		$name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		$value =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		if ($FORM{$name}) { $FORM{$name} = "$FORM{$name}, $value"; }
		else { $FORM{$name} = $value; }
	}
}
elsif ($ENV{'QUERY_STRING'} =~ /terms=([^\s&;\?]*)/i) {
	$FORM{'terms'} = $1;
	$FORM{'terms'} =~ s/\+/ /g;
	if ($ENV{'QUERY_STRING'} =~ /boolean=all/i) {
		$FORM{'boolean'} = "all terms";
	}
}
else {
	$ListOnly = 1;
}

if (%webbbs4_dirs || $searchindex) {
	$DisplayByDate = 1;
	$FORM{'case'} = "insensitive";
	if ($FORM{'boolean'} eq 'as a phrase') {
		$FORM{'boolean'} = "all terms";
	}
}

unless ($FORM{'boolean'}) { $FORM{'boolean'} = "any terms"; }
unless ($FORM{'case'}) { $FORM{'case'} = "insensitive"; }
unless ($FORM{'hits'}) { $FORM{'hits'} = 25; }
unless ($FORM{'terms'}) { $NoTerms = 1; }

$matchstring = "\\w\\.\\-\\'";
foreach $extrachar (keys %extrachars) {
	$matchstring .= "\\$extrachars{$extrachar}";
}

if ($FORM{'case'} eq "insensitive") {
	foreach $extrachar (keys %extrachars) {
		unless ($extrachar eq $extrachars{$extrachar}) {
			$FORM{'terms'} =~ s/$extrachars{$extrachar}/$extrachar/g;
		}
	}
	$FORM{'terms'} =~ tr/A-Z/a-z/;
	foreach $extrachar (keys %extrachars) {
		unless ($extrachar eq $extrachars{$extrachar}) {
			$FORM{'terms'} =~ s/$extrachar/$extrachars{$extrachar}/g;
		}
	}
}
$FORM{'terms'} =~ s/\s+/ /g;
$FORM{'terms'} =~ s/^\s//;
$FORM{'terms'} =~ s/\s$//;
$FORM{'terms'} =~ s/[^$matchstring]/ /g;
$FORM{'terms'} =~ s/([^\w\s])/\\$1/g;

if ($FORM{'boolean'} eq "as a phrase") { push (@terms,$FORM{'terms'}); }
else { @terms = split(/\s+/,$FORM{'terms'}); }

$matchcount=0;
$filecount=0;

unless ($searchindex && !($ListOnly)) {
	foreach $file (@dirs) {
		undef (@AllFiles);
		undef ($AllText);
		if ($file =~ s/\+$//) {
			$AllText = 1;
		}
		if ($file =~ s/\*$//) {
			$AllDirs = 1;
			&find ($file);
		}
		else {
			opendir(DIR,$file);
			@AllFiles = readdir(DIR);
			closedir(DIR);
		}
		$file =~ s/\/$//;
		foreach $subfile (@AllFiles) {
			unless ($subfile =~ /^$file/) {
				$subfile = $file."/".$subfile;
			}
			if ((-T "$subfile")
			  && ($AllText || ($subfile =~ /\.(s|p)*htm(l)*$/i))
			  && (!$avoid || ($subfile !~ /$avoid/))) {
				$kbytesize{$subfile} = int((((stat($subfile))[7])/1024)+.5);
				$kbytestotal += $kbytesize{$subfile};
				push (@files,"$subfile");
			}
		}
	}
}

print "Content-type: text/html\n\n";

if ($ListOnly) {
	if ($searchindex) {
		unlink "$searchindex";
		open (SEARCH,">$searchindex") || &Error_SearchIndex;
	}
	@FILES = sort (@files);
	&Header("File List");
	print "<PRE>\n\n";
	print "The following files are included in the search:\n\n";
	foreach $FILE (@FILES) {
		if ($searchindex) {
			undef $string;
			$title=$poster=$description="";
			open (FILE,"$FILE");
			@LINES = <FILE>;
			close (FILE);
			$mtime = (stat($FILE))[9];
			$kbytesize = int((((stat($FILE))[7])/1024)+.5);
			$string = join(' ',@LINES);
			undef @LINES;
			$string =~ s/\n/ /g;
			$string =~ s/<SCRIPT.*<\/SCRIPT>/ /gi;
			if ($string =~ /<TITLE>([^>]+)<\/TITLE>/i) {
				$title = "$1";
			}
			elsif ($string =~ /SUBJECT>(.+)POSTER>(.+)EMAIL>/i) {
				$title = "$1";
				$poster = "$2";
			}
			else {
				$title = "$FILE";
			}
			if ($SplitNames) {
				@names = split (/<\s*A\s*NAME\s*=\s*"*/i,$string);
			}
			else {
				$names[0] = $string;
			}
			$namescount = @names;
			foreach $key (0..$namescount-1) {
				unless ($key==0) {
					$filename = $names[$key];
					$filename =~ s/^([^">]*).*/#$1/;
					$names[$key] =~ s/^[^">]*"*\s*>(.*)/$1/;
					$filename = "$FILE"."$filename";
				}
				else {
					$filename = $FILE;
				}
				$string = $names[$key];
				unless (!($UseDescs) || ($NoMETAs > 0)) {
					if ($string =~ /<[^>]*META[^>]+NAME\s*=[ "]*description[ "]+CONTENT\s*=\s*"(([^>"])*)"[^>]*>/i) {
						$description = "$1";
					}
				}
				$title =~ s/\s+/ /g;
				$title =~ s/^\s*//;
				$title =~ s/\s*$//;
				$title =~ s/"/'/;
				if ($poster) {
					$poster =~ s/\s+/ /g;
					$poster =~ s/^\s*//;
					$poster =~ s/\s*$//;
					$poster =~ s/"/'/;
				}
				$string =~ s/<[^>]*\s+ALT\s*=\s*"(([^>"])*)"[^>]*>/$1/ig;
				unless ($NoMETAs > 0) {
					$string =~ s/<[^>]*META[^>]+NAME\s*=[ "]*(description|keywords)[ "]+CONTENT\s*=\s*"(([^>"])*)"[^>]*>/$2/ig;
				}
				unless (!($UseDescs) || ($description{$filename})) {
					$description = $string;
					if ($description =~ /<BODY/) {
						$description =~ s/.*<BODY[^>]*>(.*)/$1/i;
					}
					elsif ($description =~ /LINKURL>/) {
						$description =~ s/.*LINKURL>[^ ]*(.*)/$1/i;
					}
					$description =~ s/<([^>])*>//g;
					$description =~ s/\s+/ /g;
					$description =~ s/^\s*//;
					$description =~ s/\s*$//;
					$description =~ s/"/'/;
					$description = substr($description,0,$DescLength);
				}
				$string =~ s/<([^>])*>//g;
				$string =~ tr/A-Z/a-z/;
				foreach $extrachar (keys %extrachars) {
					unless ($extrachar eq $extrachars{$extrachar}) {
						$string =~ s/$extrachar/$extrachars{$extrachar}/g;
					}
				}
				$string =~ s/&[^;\s]*;/ /g;
				$string =~ s/[^$matchstring]/ /g;
				$string =~ s/(\s)+/ /g;
				%wordlist = ();
				print SEARCH "$filename \"$title\" \"$poster\" \"$description\" $mtime $kbytesize ";
				@words = split (/\s/,$string);
				foreach $word (@words) {
					next if ($wordlist{$word});
					$wordlist{$word} = 1;
					print SEARCH "$word ";
				}
				print SEARCH "\n";
			}
		}
		if (%otherurls) {
			foreach $path (keys %otherurls) {
				$FILE =~ s/$path/$otherurls{$path}/i;
			}
		}
		$FILE =~ s/$basepath/$baseurl/i;
		print "$FILE\n";
		$filecount++;
		next;
	}
	print "\nTotal:  ",&commas($filecount)," files\n";
	print "        ",&commas($kbytestotal)," kb\n";
	if ($searchindex) {
		close (SEARCH,">$searchindex");
		print "\nSearch index file successfully created.\n";
	}
	if (%webbbs4_dirs) {
		print "\nForums:\n";
		foreach $key (keys %webbbs4_dirs) {
			if ($DBMType==1) {
				tie (%MessageList,'AnyDBM_File',"$key/messagelist",O_RD,0666,$DB_HASH);
			}
			elsif ($DBMType==2) {
				dbmopen(%MessageList,"$key/messagelist",0666);
			}
			else {
				tie (%MessageList,'AnyDBM_File',"$key/messagelist",O_RD,0666);
			}
			@messages = (keys %MessageList);
			$TotalMessages = @messages;
			print "$webbbs4_dirs{$key} (",&commas($TotalMessages)," messages)\n";
			if ($DBMType==2) { dbmclose (%MessageList); }
			else { untie %MessageList; }
		}
	}
	print "\n</PRE>\n";
	&PrintForm;
	&Footer;
	exit;
}

if ($AllowDateSearch && ($FORM{'DateRange'} eq "Range")) {
	unless ($FORM{'StartDateA'}) { $FORM{'StartDateA'} = 1; }
	unless ($FORM{'EndDateA'}) { $FORM{'EndDateA'} = 15; }
	if ($FORM{'StartDateC'} < 1990) { $FORM{'StartDateC'} = 1990; }
	if ($FORM{'EndDateC'} < $FORM{'StartDateC'}) { $FORM{'EndDateC'} = $FORM{'StartDateC'}; }
	$searchrange = "Files posted or updated between $FORM{'StartDateA'} $month[$FORM{'StartDateB'}] ";
	$searchrange .= "$FORM{'StartDateC'} and $FORM{'EndDateA'} ";
	$searchrange .= "$month[$FORM{'EndDateB'}] $FORM{'EndDateC'}";
	$startday = &rangedate($FORM{'StartDateB'}+1,$FORM{'StartDateA'},$FORM{'StartDateC'}-1900);
	$endday = &rangedate($FORM{'EndDateB'}+1,$FORM{'EndDateA'}+1,$FORM{'EndDateC'}-1900);
}
else {
	$startday = 0;
	$endday = 9999999999;
}

unless ($NoTerms) {
	if ($searchindex) {
		open (SEARCH,"$searchindex");
		while (<SEARCH>) {
			if (/^(\S+) "(.*)" "(.*)" "(.*)" (\d+) (\d+) (.*)/) {
				$FILE = $1;
				$title{$FILE} = $2;
				$poster{$FILE} = $3;
				$description{$FILE} = $4;
				$mtime = $mtime{$FILE} = $5;
				$kbytesize{$FILE} = $6;
				$string = $7;
				$filecount++;
				if (($mtime >= $startday) && ($mtime <= $endday)) {
					$value = 0;
					if ($FORM{'boolean'} eq 'all terms') {
						foreach $term (@terms) {
							$test = ($string =~ s/$term//ig);
							if ($test < 1) {
								$value = 0;
								last;
							}
							else {
								$value = $value+$test;
							}
						}
					}
					else {
						foreach $term (@terms) {
							$test = ($string =~ s/$term//ig);
							$value = $value+$test;
						}
					}
					if ($value > 0) {
						$matchcount++;
						$update{$FILE} = &Get_Date;
						$truval{$FILE} = 1;
					}
				}
			}
		}
		close (SEARCH);
	}
	else {
		foreach $FILE (@files) {
			undef $string;
			open (FILE,"$FILE");
			@LINES = <FILE>;
			close (FILE);
			$filecount++;
			$mtime = $mtime{$FILE} = (stat($FILE))[9];
			next unless (($mtime >= $startday) && ($mtime <= $endday));
			$update{$FILE} = &Get_Date;
			$string = join(' ',@LINES);
			undef @LINES;
			$string =~ s/\n/ /g;
			$string =~ s/<SCRIPT.*<\/SCRIPT>/ /gi;
			if ($string =~ /<TITLE>([^>]+)<\/TITLE>/i) {
				$title{$FILE} = "$1";
			}
			elsif ($string =~ /SUBJECT>(.+)POSTER>(.+)EMAIL>/i) {
				$title{$FILE} = "$1";
				$poster{$FILE} = "$2";
			}
			else {
				$title{$FILE} = "$FILE";
			}
			if ($SplitNames) {
				@names = split (/<\s*A\s*NAME\s*=\s*"*/i,$string);
			}
			else {
				$names[0] = $string;
			}
			$namescount = @names;
			foreach $key (0..$namescount-1) {
				unless ($key==0) {
					$filename = $names[$key];
					$filename =~ s/^([^">]*).*/#$1/;
					$names[$key] =~ s/^[^">]*"*\s*>(.*)/$1/;
					$filename = "$FILE"."$filename";
				}
				else {
					$filename = $FILE;
				}
				$kbytesize{$filename} = $kbytesize{$FILE};
				$mtime{$filename} = $mtime{$FILE};
				$update{$filename} = $update{$FILE};
				$title{$filename} = $title{$FILE};
				$poster{$filename} = $poster{$FILE};
				$val{$filename} = 0;
				$string = $names[$key];
				unless (!($UseDescs) || ($NoMETAs > 0)) {
					if ($string =~ /<[^>]*META[^>]+NAME\s*=[ "]*description[ "]+CONTENT\s*=\s*"(([^>"])*)"[^>]*>/i) {
						$description{$filename} = "$1";
					}
				}
				$title{$filename} =~ s/\s+/ /g;
				$title{$filename} =~ s/^\s*//;
				$title{$filename} =~ s/\s*$//;
				if ($poster{$filename}) {
					$poster{$filename} =~ s/\s+/ /g;
					$poster{$filename} =~ s/^\s*//;
					$poster{$filename} =~ s/\s*$//;
				}
				$string =~ s/<[^>]*\s+ALT\s*=\s*"(([^>"])*)"[^>]*>/$1/ig;
				unless ($NoMETAs > 0) {
					$string =~ s/<[^>]*META[^>]+NAME\s*=[ "]*(description|keywords)[ "]+CONTENT\s*=\s*"(([^>"])*)"[^>]*>/$2/ig;
				}
				unless (!($UseDescs) || ($description{$filename})) {
					$description = $string;
					if ($description =~ /<BODY/) {
						$description =~ s/.*<BODY[^>]*>(.*)/$1/i;
					}
					elsif ($description =~ /LINKURL>/) {
						$description =~ s/.*LINKURL>[^ ]*(.*)/$1/i;
					}
					$description =~ s/<([^>])*>//g;
					$description =~ s/\s+/ /g;
					$description =~ s/^\s*//;
					$description =~ s/\s*$//;
					$description{$filename} = substr($description,0,$DescLength);
					undef $description;
				}
				$string =~ s/<([^>])*>//g;
				if ($FORM{'case'} eq "insensitive") {
					$string =~ tr/A-Z/a-z/;
				}
				foreach $extrachar (keys %extrachars) {
					unless ($extrachar eq $extrachars{$extrachar}) {
						$string =~ s/$extrachar/$extrachars{$extrachar}/g;
					}
				}
				$string =~ s/&[^;\s]*;/ /g;
				$string =~ s/[^$matchstring]/ /g;
				$string =~ s/(\s)+/ /g;
				if ($FORM{'boolean'} eq 'all terms') {
					foreach $term (@terms) {
						if ($FORM{'case'} eq 'insensitive') {
							$test = ($string =~ s/$term//ig);
							if ($test < 1) {
								$val{$filename} = 0;
								last;
							}
							else {
								$val{$filename} = $val{$filename}+$test;
							}
						}
						elsif ($FORM{'case'} eq 'sensitive') {
							$test = ($string =~ s/$term//g);
							if ($test < 1) {
								$val{$filename} = 0;
								last;
							}
							else {
								$val{$filename} = $val{$filename}+$test;
							}
						}
					}
				}
				else {
					foreach $term (@terms) {
						if ($FORM{'case'} eq 'insensitive') {
							$test = ($string =~ s/$term//ig);
						}
						elsif ($FORM{'case'} eq 'sensitive') {
							$test = ($string =~ s/$term//g);
						}
						$val{$filename} = $val{$filename}+$test;
					}
				}
				if ($val{$filename} > 0) {
					$truval{$filename} = ($val{$filename});
					$matchcount++;
				}
			}
		}
	}
	if (%webbbs4_dirs) {
		foreach $key (keys %webbbs4_dirs) {
			if ($DBMType==1) {
				tie (%MessageList,'AnyDBM_File',"$key/messagelist",O_RD,0666,$DB_HASH);
			}
			elsif ($DBMType==2) {
				dbmopen(%MessageList,"$key/messagelist",0666);
			}
			else {
				tie (%MessageList,'AnyDBM_File',"$key/messagelist",O_RD,0666);
			}
			open (SEARCH,"$key/searchterms.idx");
			while (<SEARCH>) {
				if (/^(\d+) (.*)/) {
					$message = $1;
					$string = $2;
					$filecount++;
					if ((int($MessageList{$message}) >= $startday) && (int($MessageList{$message}) <= $endday)) {
						$value = 0;
						if ($FORM{'boolean'} eq 'all terms') {
							foreach $term (@terms) {
								$test = ($string =~ s/$term//ig);
								if ($test < 1) {
									$value = 0;
									last;
								}
								else {
									$value = $value+$test;
								}
							}
						}
						else {
							foreach $term (@terms) {
								$test = ($string =~ s/$term//ig);
								$value = $value+$test;
							}
						}
						if ($value > 0) {
							$matchcount++;
							$FILE = $webbbs4_dirs{$key}."?review=$message";
							($date,$sub,$poster,$prev,$next,$count,$admin,$ip) =
							  split(/\|/,$MessageList{$message});
							$mtime = $mtime{$FILE} = $date;
							$update{$FILE} = &Get_Date;
							$truval{$FILE} = 1;
							$title{$FILE} = $sub;
							$poster{$FILE} = $poster;
							$title{$FILE} =~ s/<[^>]*\s+ALT\s*=\s*"(([^>"])*)"[^>]*>/$1/ig;
							$poster{$FILE} =~ s/<[^>]*\s+ALT\s*=\s*"(([^>"])*)"[^>]*>/$1/ig;
							$subdir = "bbs".int($message/1000);
							$webbbs4{$FILE} = "$key/$subdir/$message";
						}
					}
				}
			}
			close (SEARCH);
			if ($DBMType==2) { dbmclose (%MessageList); }
			else { untie %MessageList; }
		}
	}
}

foreach $term (@terms) {
	$term =~ s/\\\\/BaCkSlAsH/g;
	$term =~ s/\\//g;
	$term =~ s/BaCkSlAsH/\\/g;
	$term =~ s/&/&amp;/g;
	$term =~ s/"/&quot;/g;
	$term =~ s/>/&gt;/g;
	$term =~ s/</&lt;/g;
}

unless ($FORM{'first'}) { $FORM{'first'} = 1; }
unless ($FORM{'last'}) { $FORM{'last'} = $FORM{'hits'}; }

&NavBar;

&Header("Search Results");

print "<P ALIGN=CENTER><BIG><BIG><STRONG>Search Results</STRONG></BIG></BIG>\n";
print "<P ALIGN=CENTER>Keywords ($FORM{'boolean'}, ";
print "case $FORM{'case'}): <STRONG>";
if ($NoTerms) { print "No Search Terms Provided!"; }
else { foreach $term (@terms) { print "$term "; } }
print "</STRONG>\n";
if ($searchrange) { print "<BR>$searchrange\n"; }
print "<P ALIGN=CENTER><SMALL>";
print "(<STRONG>",&commas($filecount),"</STRONG> files searched; ";
print "<STRONG>",&commas($matchcount),"</STRONG> match";
if ($matchcount == 1) { print " found)"; }
else { print "es found)"; }
print "</SMALL>\n";

if ($matchcount == 0) {
	print "<P ALIGN=CENTER>No documents matched your search criteria!";
	print "<BR>You might want to revise them and try again.\n";
}
else {
	print $NavBar;
	$Count = 0;
	print "<P><DL>\n";
	if ($DisplayByDate > 0) {
		foreach $key (sort ByDate keys %truval) {
			&PrintEntry;
		}
	}
	else {
		foreach $key (sort ByValue keys %truval) {
			&PrintEntry;
		}
	}
	print "</DL>\n";
	print $NavBar;
}

&PrintForm;
&Footer;
exit;

sub commas {
	local($_)=@_;
	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

sub Get_Date {
	$gd_time = $mtime+($HourOffset*3600);
	($mday,$mon,$yr) = (localtime($gd_time))[3,4,5];
	$yr += 1900;
	$date = "$mday $month[$mon] $yr";
	return $date;
}

sub PrintEntry {
	$Count++;
	next if ($Count < $FORM{'first'});
	last if ($Count > $FORM{'last'});
	if ($webbbs4{$key}) {
		$kbytesize{$key} = int((((stat($webbbs4{$key}))[7])/1024)+.5);
		if ($UseDescs) {
			open (WEBBBS4,"$webbbs4{$key}");
			@message = <WEBBBS4>;
			close (WEBBBS4);
			$description = join(' ',@message);
			$description =~ s/\n/ /g;
			$description =~ s/.*LINKURL>[^ ]*(.*)/$1/i;
			$description =~ s/<([^>])*>//g;
			$description =~ s/\s+/ /g;
			$description =~ s/^\s*//;
			$description =~ s/\s*$//;
			$description{$key} = substr($description,0,$DescLength);
		}
	}
	$fileurl = $key;
	if (%otherurls) {
		foreach $path (keys %otherurls) {
			$fileurl =~ s/$path/$otherurls{$path}/i;
			$title{$key} =~ s/$path/$otherurls{$path}/i;
		}
	}
	$fileurl =~ s/$basepath/$baseurl/i;
	$title{$key} =~ s/$basepath/$baseurl/i;
	print "<DT>$Count. <STRONG><A HREF=\"$fileurl\">";
	print "$title{$key}</A></STRONG>";
	if ($poster{$key}) { print " (message posted by $poster{$key})"; }
	print "<DD><SMALL>";
	unless ($DisplayByDate > 0) {
		print "Keyword Matches: ";
		print "<STRONG>$truval{$key}</STRONG>; ";
	}
	print "Size: <STRONG>$kbytesize{$key} kb</STRONG>; ";
	if ($poster{$key}) { print "Posted "; }
	else { print "Last updated "; }
	print "<STRONG>$update{$key}</STRONG></SMALL>\n";
	if ($description{$key}) {
		print "<BR><EM>$description{$key}</EM>\n";
	}
	print "<P>\n";
}

sub ByDate {
	$aval = $mtime{$a};
	$bval = $mtime{$b};
	$bval <=> $aval;
}

sub ByValue {
	$aval = $truval{$a};
	$bval = $truval{$b};
	$bval <=> $aval;
}

sub wanted {
	(push (@AllFiles, $name)) && -f $_;
}

sub PrintForm {
	$PrintForm = "<P><CENTER><TABLE BORDER CELLPADDING=12><TR><TD>\n";
	$PrintForm .= "<CENTER><BIG><STRONG>New Search</STRONG></BIG>\n";
	$PrintForm .= "<FORM METHOD=POST ACTION=\"$cgiurl\">\n";
	$PrintForm .= "<P><STRONG>Terms for which to search:</STRONG>\n";
	$PrintForm .= "<BR><INPUT TYPE=TEXT NAME=\"terms\" SIZE=60 VALUE=\"";
	foreach $term (@terms) {
		$PrintForm .= "$term ";
	}
	$PrintForm .= "\">\n<P><STRONG>Find:</STRONG> <SELECT NAME=\"boolean\"><OPTION";
	if ($FORM{'boolean'} eq 'any terms') { $PrintForm .= " SELECTED"; }
	$PrintForm .= ">any terms<OPTION";
	if ($FORM{'boolean'} eq 'all terms') { $PrintForm .= " SELECTED"; }
	$PrintForm .= ">all terms";
	unless (%webbbs4_dirs || $searchindex) {
		$PrintForm .= "<OPTION";
		if ($FORM{'boolean'} eq 'as a phrase') { $PrintForm .= " SELECTED"; }
		$PrintForm .= ">as a phrase";
		$PrintForm .= "</SELECT> ";
		$PrintForm .= "<STRONG>Case:</STRONG> <SELECT NAME=\"case\"><OPTION";
		if ($FORM{'case'} eq 'insensitive') { $PrintForm .= " SELECTED"; }
		$PrintForm .= ">insensitive<OPTION";
		if ($FORM{'case'} eq 'sensitive') { $PrintForm .= " SELECTED"; }
		$PrintForm .= ">sensitive";
	}
	$PrintForm .= "</SELECT> ";
	$PrintForm .= "<STRONG>Display:</STRONG> <SELECT NAME=\"hits\"> ";
	$PrintForm .= "<OPTION";
	if ($FORM{'hits'} == 10) { $PrintForm .= " SELECTED"; }
	$PrintForm .= ">10<OPTION";
	if ($FORM{'hits'} == 25) { $PrintForm .= " SELECTED"; }
	$PrintForm .= ">25<OPTION";
	if ($FORM{'hits'} == 50) { $PrintForm .= " SELECTED"; }
	$PrintForm .= ">50<OPTION";
	if ($FORM{'hits'} == 100) { $PrintForm .= " SELECTED"; }
	$PrintForm .= ">100</SELECT> hits per page\n";
	if ($AllowDateSearch) {
		$PrintForm .= "<P><STRONG>Search:</STRONG>\n";
		$PrintForm .= "<BR><INPUT TYPE=RADIO NAME=DateRange VALUE=All";
		unless ($FORM{'DateRange'} eq "Range") {
			$PrintForm .= " CHECKED";
			$mday=$mon="";
		}
		$PrintForm .= "> All files";
		$time = time;
		if (($FORM{'DateRange'} eq "Range") && $FORM{'StartDateA'}) {
			$mday = $FORM{'StartDateA'};
			$mon = $FORM{'StartDateB'};
			$year = $FORM{'StartDateC'}-1900;
		}
		else { $year = (localtime($time))[5]; }
		$year+=1900;
		$PrintForm .= "<BR><INPUT TYPE=RADIO NAME=DateRange VALUE=Range";
		if ($FORM{'DateRange'} eq "Range") { $PrintForm .= " CHECKED"; }
		$PrintForm .= "> Files posted or updated between<BR>";
		$PrintForm .= "<FONT FACE=\"Courier\"><INPUT TYPE=TEXT NAME=\"StartDateA\" SIZE=2";
		if ($mday) { $PrintForm .= " VALUE=$mday"; }
		$PrintForm .= "> ";
		$PrintForm .= "<SELECT NAME=\"StartDateB\">";
		foreach $key (0..11) {
			$PrintForm .= "<OPTION VALUE=\"$key\"";
			if ($key == $mon) { $PrintForm .= " SELECTED"; }
			$PrintForm .= ">$month[$key]";
		}
		$PrintForm .= "</SELECT> ";
		$PrintForm .= "<INPUT TYPE=TEXT NAME=\"StartDateC\" SIZE=4 VALUE=$year></FONT>";
		$PrintForm .= " and ";
		if (($FORM{'DateRange'} eq "Range") && $FORM{'EndDateA'}) {
			$mday = $FORM{'EndDateA'};
			$mon = $FORM{'EndDateB'};
			$year = $FORM{'EndDateC'}-1900;
		}
		else { $year = (localtime($time))[5]; }
		$year+=1900;
		$PrintForm .= "<FONT FACE=\"Courier\"><INPUT TYPE=TEXT NAME=\"EndDateA\" SIZE=2";
		if ($mday) { $PrintForm .= " VALUE=$mday"; }
		$PrintForm .= "> ";
		$PrintForm .= "<SELECT NAME=\"EndDateB\">";
		foreach $key (0..11) {
			$PrintForm .= "<OPTION VALUE=\"$key\"";
			if ($key == $mon) { $PrintForm .= " SELECTED"; }
			$PrintForm .= ">$month[$key]";
		}
		$PrintForm .= "</SELECT> ";
		$PrintForm .= "<INPUT TYPE=TEXT NAME=\"EndDateC\" SIZE=4 VALUE=$year></FONT>\n";
	}
	$PrintForm .= "<P><INPUT TYPE=SUBMIT VALUE=\"Search\">";
	$PrintForm .= "</CENTER></FORM>\n<SMALL>\n";
	if ($FormExplanation) { $PrintForm .= "<P ALIGN=CENTER>$FormExplanation\n"; }
	$PrintForm .= "<P ALIGN=CENTER>";
	$PrintForm .= "Maintained with <STRONG>";
	$PrintForm .= "<A HREF=\"http://awsd.com/scripts/websearch/\">";
	$PrintForm .= "WebSearch $version</A></STRONG>.\n</SMALL>\n";
	$PrintForm .= "</TD></TR></TABLE></CENTER>\n";
	if ($ListOnly) {
		$PrintForm =~ s/New Search/Site Search/g;
		print $PrintForm;
		$PrintForm =~ s/&/&amp;/g;
		$PrintForm =~ s/</&lt;/g;
		$PrintForm =~ s/>/&gt;/g;
		$PrintForm =~ s/"/&quot;/g;
		$PrintForm =~ s/\n/<BR>/g;
		print "<P><TABLE><TR><TD>";
		print "<STRONG>Copy the following HTML code onto any pages ";
		print "on which you want a search form like the above:</STRONG>";
		print "</TD></TR><TR><TD NOWRAP>$PrintForm</TD></TR></TABLE>\n";	
	}
	elsif ($PrintNewForm) {
		print $PrintForm;
	}
}

sub NavBar {
	$NavBar = "<P><CENTER><TABLE><TR><TD><STRONG>Matches:</STRONG></TD>";
	$prevstart = ($FORM{'first'}-$FORM{'hits'});
	$prevend = ($FORM{'last'}-$FORM{'hits'});
	if ($prevstart > 1) {
		$NavBar .= "<FORM METHOD=POST ACTION=\"$cgiurl\"><TD>";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"terms\" VALUE=\"";
		foreach $term (@terms) { $NavBar .= "$term "; }
		$NavBar .= "\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"boolean\" ";
		$NavBar .= "VALUE=\"$FORM{'boolean'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"case\" ";
		$NavBar .= "VALUE=\"$FORM{'case'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"hits\" ";
		$NavBar .= "VALUE=\"$FORM{'hits'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"first\" ";
		$NavBar .= "VALUE=\"1\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"last\" ";
		$NavBar .= "VALUE=\"$FORM{'hits'}\">";
		if ($FORM{'DateRange'} eq "Range") {
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"DateRange\" ";
			$NavBar .= "VALUE=\"$FORM{'DateRange'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateC'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateC'}\">";
		}
		$NavBar .= "<INPUT TYPE=SUBMIT ";
		$NavBar .= "VALUE=\"1 - $FORM{'hits'}\">";
		$NavBar .= "</TD></FORM>";
	}
	if ($FORM{'first'} > 1) {
		$NavBar .= "<FORM METHOD=POST ACTION=\"$cgiurl\"><TD>";
		if (($prevstart-$FORM{'hits'})>1) { $NavBar .= "<STRONG>...</STRONG></TD><TD>"; }
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"terms\" VALUE=\"";
		foreach $term (@terms) { $NavBar .= "$term "; }
		$NavBar .= "\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"boolean\" ";
		$NavBar .= "VALUE=\"$FORM{'boolean'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"case\" ";
		$NavBar .= "VALUE=\"$FORM{'case'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"hits\" ";
		$NavBar .= "VALUE=\"$FORM{'hits'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"first\" ";
		$NavBar .= "VALUE=\"$prevstart\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"last\" ";
		$NavBar .= "VALUE=\"$prevend\">";
		if ($FORM{'DateRange'} eq "Range") {
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"DateRange\" ";
			$NavBar .= "VALUE=\"$FORM{'DateRange'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateC'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateC'}\">";
		}
		$NavBar .= "<INPUT TYPE=SUBMIT ";
		$NavBar .= "VALUE=\"$prevstart - $prevend\">";
		$NavBar .= "</TD></FORM>";
	}
	$thisend = $FORM{'last'};
	if ($thisend > $matchcount) { $thisend = $matchcount; }
	$NavBar .= "<TD><STRONG>$FORM{'first'} - $thisend</STRONG></TD>";
	$nextstart = ($FORM{'first'}+$FORM{'hits'});
	$nextend = ($FORM{'last'}+$FORM{'hits'});
	if ($FORM{'last'} < $matchcount) {
		$NavBar .= "<FORM METHOD=POST ACTION=\"$cgiurl\"><TD>";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"terms\" VALUE=\"";
		foreach $term (@terms) { $NavBar .= "$term "; }
		$NavBar .= "\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"boolean\" ";
		$NavBar .= "VALUE=\"$FORM{'boolean'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"case\" ";
		$NavBar .= "VALUE=\"$FORM{'case'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"hits\" ";
		$NavBar .= "VALUE=\"$FORM{'hits'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"first\" ";
		$NavBar .= "VALUE=\"$nextstart\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"last\" ";
		$NavBar .= "VALUE=\"$nextend\">";
		if ($FORM{'DateRange'} eq "Range") {
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"DateRange\" ";
			$NavBar .= "VALUE=\"$FORM{'DateRange'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateC'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateC'}\">";
		}
		$NavBar .= "<INPUT TYPE=SUBMIT VALUE=\"";
		if ($nextend > $matchcount) { $nextend = $matchcount; }
		$nextset = $nextend - $nextstart + 1;
		$NavBar .= "$nextstart";
		unless ($nextset == 1) { $NavBar .= " - $nextend"; }
		$NavBar .= "\"></TD></FORM>";
	}
	if ($nextend < $matchcount) {
		$finalend = (int($matchcount/$FORM{'hits'})+1)*$FORM{'hits'};
		$finalstart = $finalend-$FORM{'hits'}+1;
		$NavBar .= "<FORM METHOD=POST ACTION=\"$cgiurl\"><TD>";
		if (($finalstart-$nextend)>1) { $NavBar .= "<STRONG>...</STRONG></TD><TD>"; }
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"terms\" VALUE=\"";
		foreach $term (@terms) { $NavBar .= "$term "; }
		$NavBar .= "\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"boolean\" ";
		$NavBar .= "VALUE=\"$FORM{'boolean'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"case\" ";
		$NavBar .= "VALUE=\"$FORM{'case'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"hits\" ";
		$NavBar .= "VALUE=\"$FORM{'hits'}\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"first\" ";
		$NavBar .= "VALUE=\"$finalstart\">";
		$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"last\" ";
		$NavBar .= "VALUE=\"$finalend\">";
		if ($FORM{'DateRange'} eq "Range") {
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"DateRange\" ";
			$NavBar .= "VALUE=\"$FORM{'DateRange'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"StartDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'StartDateC'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateA\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateA'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateB\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateB'}\">";
			$NavBar .= "<INPUT TYPE=HIDDEN NAME=\"EndDateC\" ";
			$NavBar .= "VALUE=\"$FORM{'EndDateC'}\">";
		}
		$NavBar .= "<INPUT TYPE=SUBMIT VALUE=\"";
		if ($finalend > $matchcount) { $finalend = $matchcount; }
		$finalset = $finalend - $finalstart + 1;
		$NavBar .= "$finalstart";
		unless ($finalset == 1) { $NavBar .= " - $finalend"; }
		$NavBar .= "\"></TD></FORM>";
	}
	$NavBar .= "</TR></TABLE></CENTER>\n";
}

sub rangedate {
	($perp_mon,$perp_day,$perp_year) = @_;
	%day_counts =
	  (1,0,2,31,3,59,4,90,5,120,6,151,7,181,
	  8,212,9,243,10,273,11,304,12,334);
	$perp_days = (($perp_year-69)*365)+(int(($perp_year-69)/4));
	$perp_days += $day_counts{$perp_mon};
	if ((int(($perp_year-68)/4) eq (($perp_year-68)/4))
	  && ($perp_mon>2)) {
		$perp_days++;
	}
	$perp_days += $perp_day;
	$perp_days -= 366;
	$perp_secs = ($perp_days*86400)+18000;
	$hour = (localtime($perp_secs))[2];
	if ($hour>0) { $perp_secs-=3600; }
	$perp_secs -= ($HourOffset*3600);
	return $perp_secs;
}

sub Header {
	($title) = @_;
	print "<HTML><HEAD><TITLE>$title</TITLE>\n";
	if ($meta_file) {
		open (HEADLN,"$meta_file");
		@headln = <HEADLN>;
		close (HEADLN);
		foreach $line (@headln) {
			print "$line";
		}
	}
	print "</HEAD>\n";
	print "<BODY $bodyspec>\n";
	if ($header_file) {
		open (HEADER,"$header_file");
		@header = <HEADER>;
		close (HEADER);
		foreach $line (@header) {
			if ($line =~ /<!--#include\s+(virtual|file)\s*=\s*"*([^"\s]*)"*\s*-->/i) {
				$SSIFile = $SSIRootDir.$2;
				open (SSIFILE,"<$SSIFile");
				while (<SSIFILE>) { print "$_"; }
				close (SSIFILE);
			}
			else { print "$line"; }
		}
	}
}

sub Footer {
	if ($footer_file) {
		open (FOOTER,"$footer_file");
		@footer = <FOOTER>;
		close (FOOTER);
		foreach $line (@footer) {
			if ($line =~ /<!--#include\s+(virtual|file)\s*=\s*"*([^"\s]*)"*\s*-->/i) {
				$SSIFile = $SSIRootDir.$2;
				open (SSIFILE,"<$SSIFile");
				while (<SSIFILE>) { print "$_"; }
				close (SSIFILE);
			}
			else { print "$line"; }
		}
	}
	print "</BODY></HTML>\n";
}

sub Error_SearchIndex {
	&Header("Error");
	print "<P>The script was unable to create the search index file. ";
	print "This is most likely either because the file's location has been ";
	print "incorrectly defined, or because the script doesn't have permission ";
	print "to write files to the designated directory.\n";
	&Footer;
	exit;
}
